<?php

namespace Drupal\jsonapi_advanced_permissions\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\Role;

/**
 * Configure JSON:API Advanced Permissions settings for this site.
 */
final class JsonApiAdvancedPermissionsSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'jsonapi_advanced_permissions_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['jsonapi_advanced_permissions.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $options = [
      'collection' => $this->t('Collection GET'),
      'collection.post' => $this->t('Collection POST'),
      'individual' => $this->t('Individual GET'),
      'individual.patch' => $this->t('Individual PATCH'),
      'individual.delete' => $this->t('Individual DELETE'),
    ];

    $form['permissions'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Permissions'),
      '#options' => $options,
      '#default_value' => $this->config('jsonapi_advanced_permissions.settings')->get('permissions') ?? [],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {

    $permissions = $form_state->getValue('permissions')
      ? array_keys(array_filter($form_state->getValue('permissions')))
      : [];
    $previous_permissions = $this->config('jsonapi_advanced_permissions.settings')
      ->get('permissions') ?? [];
    $removed_permissions = array_diff($previous_permissions, $permissions);

    $roles = Role::loadMultiple();

    foreach ($roles as $role) {
      $strings = $role->getPermissions();
      foreach ($removed_permissions as $removed_permission) {
        $permission_detail = explode('.', $removed_permission);

        $pattern = '/^jsonapi_advanced_permissions ' . ($permission_detail[1] ?? 'get') . ' on ([a-zA-Z0-9_]+)__([a-zA-Z0-9_]+) ' . $permission_detail[0] . '$/';

        foreach ($strings as $string) {
          if (preg_match($pattern, $string, $match)) {
            $role->revokePermission($string);
          }
        }
      }
      $role->save();
    }

    $this->config('jsonapi_advanced_permissions.settings')
      ->set('permissions', $permissions)
      ->save();
    parent::submitForm($form, $form_state);
  }

}
